/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef XUNFEINLPENGINE_P_H
#define XUNFEINLPENGINE_P_H

#include <functional>
#include <cpr/cpr.h>
#include <jsoncpp/json/json.h>
#include <kylin-ai/ai-engine/large-model/cloudnlpengine.h>
#include <kylin-ai/ai-engine/large-model/error.h>

class XunfeiNlpEnginePrivate
    : public ai_engine::lm::nlp::AbstractCloudNlpEngine {
public:
    XunfeiNlpEnginePrivate();
    ~XunfeiNlpEnginePrivate();
    std::string engineName() override;

    void setConfig(const std::string &config) override;

    void setChatResultCallback(
        ai_engine::lm::nlp::ChatResultCallback callback) override;

    void setContextSize(int size) override;
    void clearContext() override;

    bool initChatModule(ai_engine::lm::EngineError &error) override;
    bool chat(const std::string &message, ai_engine::lm::EngineError &error) override;
    void stopChat() override;
    bool destroyChatModule(ai_engine::lm::EngineError &error) override;

private:
    std::string appId_;
    std::string apiKey_;
    std::string secretKey_;
    std::string hostUrl_;
    std::string domain_;

    ai_engine::lm::nlp::ChatResultCallback chatResultCallback_;

    int contextSize_{-1};
    Json::Value context_;
    void addContext_(const std::string &role, const std::string &message);
    void removeLastContext_();
    std::vector<Json::Value> streamChatData_;

    bool connectWebsocket(std::string hostUrl);
    friend void onWebSocket(bool (*doReceiveData)(XunfeiNlpEnginePrivate* self), XunfeiNlpEnginePrivate* self);
    void closeWebsocket();

    CURL *currentCurl_{nullptr};
    ai_engine::lm::EngineError currentError_;
    bool currentStatus();

    bool initResources();
    void releaseResources();

    bool doSendData(const char *data, int dataLength, int type);
    bool doReceiveChatData();

    bool isStopped_{false};

    static const std::string systemRole_;
};

#endif // XUNFEINLPENGINE_P_H
